@@ -100,14 +100,14 @@ pub unsafe extern "C" fn destructor_callback<F>(raw_cb: *mut c_void)
 where
     F: CompactionFilter,
 {
-    drop(Box::from_raw(raw_cb as *mut F));
+    drop(unsafe { Box::from_raw(raw_cb as *mut F) });
 }
 
 pub unsafe extern "C" fn name_callback<F>(raw_cb: *mut c_void) -> *const c_char
 where
     F: CompactionFilter,
 {
-    let cb = &*(raw_cb as *mut F);
+    let cb = unsafe { &*(raw_cb as *mut F) };
     cb.name().as_ptr()
 }
 
@@ -127,17 +127,17 @@ where
 {
     use self::Decision::{Change, Keep, Remove};
 
-    let cb = &mut *(raw_cb as *mut F);
-    let key = slice::from_raw_parts(raw_key as *const u8, key_length);
-    let oldval = slice::from_raw_parts(existing_value as *const u8, value_length);
+    let cb = unsafe { &mut *(raw_cb as *mut F) };
+    let key = unsafe { slice::from_raw_parts(raw_key as *const u8, key_length) };
+    let oldval = unsafe { slice::from_raw_parts(existing_value as *const u8, value_length) };
     let result = cb.filter(level as u32, key, oldval);
     match result {
         Keep => 0,
         Remove => 1,
         Change(newval) => {
-            *new_value = newval.as_ptr() as *mut c_char;
-            *new_value_length = newval.len() as size_t;
-            *value_changed = 1_u8;
+            unsafe { *new_value = newval.as_ptr() as *mut c_char };
+            unsafe { *new_value_length = newval.len() as size_t };
+            unsafe { *value_changed = 1_u8 };
             0
         }
     }
@@ -158,7 +158,11 @@ fn test_filter(level: u32, key: &[u8], value: &[u8]) -> Decision {
 fn compaction_filter_test() {
     use crate::{Options, DB};
 
-    let path = "_rust_rocksdb_filter_test";
+    let tempdir = tempfile::Builder::new()
+        .prefix("_rust_rocksdb_filter_test")
+        .tempdir()
+        .expect("Failed to create temporary path for the _rust_rocksdb_filter_test");
+    let path = tempdir.path();
     let mut opts = Options::default();
     opts.create_if_missing(true);
     opts.set_compaction_filter("test", test_filter);
