@@ -30,14 +30,14 @@ pub unsafe extern "C" fn destructor_callback<F>(raw_self: *mut c_void)
 where
     F: CompactionFilterFactory,
 {
-    drop(Box::from_raw(raw_self as *mut F));
+    drop(unsafe { Box::from_raw(raw_self as *mut F) });
 }
 
 pub unsafe extern "C" fn name_callback<F>(raw_self: *mut c_void) -> *const c_char
 where
     F: CompactionFilterFactory,
 {
-    let self_ = &*(raw_self as *const c_void as *const F);
+    let self_ = unsafe { &*(raw_self.cast_const() as *const F) };
     self_.name().as_ptr()
 }
 
@@ -52,9 +52,10 @@ pub struct CompactionFilterContext {
 
 impl CompactionFilterContext {
     unsafe fn from_raw(ptr: *mut ffi::rocksdb_compactionfiltercontext_t) -> Self {
-        let is_full_compaction = ffi::rocksdb_compactionfiltercontext_is_full_compaction(ptr) != 0;
+        let is_full_compaction =
+            unsafe { ffi::rocksdb_compactionfiltercontext_is_full_compaction(ptr) } != 0;
         let is_manual_compaction =
-            ffi::rocksdb_compactionfiltercontext_is_manual_compaction(ptr) != 0;
+            unsafe { ffi::rocksdb_compactionfiltercontext_is_manual_compaction(ptr) } != 0;
 
         Self {
             is_full_compaction,
@@ -70,18 +71,20 @@ pub unsafe extern "C" fn create_compaction_filter_callback<F>(
 where
     F: CompactionFilterFactory,
 {
-    let self_ = &mut *(raw_self as *mut F);
-    let context = CompactionFilterContext::from_raw(context);
+    let self_ = unsafe { &mut *(raw_self as *mut F) };
+    let context = unsafe { CompactionFilterContext::from_raw(context) };
     let filter = Box::new(self_.create(context));
 
     let filter_ptr = Box::into_raw(filter);
 
-    ffi::rocksdb_compactionfilter_create(
-        filter_ptr as *mut c_void,
-        Some(compaction_filter::destructor_callback::<F::Filter>),
-        Some(compaction_filter::filter_callback::<F::Filter>),
-        Some(compaction_filter::name_callback::<F::Filter>),
-    )
+    unsafe {
+        ffi::rocksdb_compactionfilter_create(
+            filter_ptr as *mut c_void,
+            Some(compaction_filter::destructor_callback::<F::Filter>),
+            Some(compaction_filter::filter_callback::<F::Filter>),
+            Some(compaction_filter::name_callback::<F::Filter>),
+        )
+    }
 }
 
 #[cfg(test)]
@@ -122,7 +125,11 @@ mod tests {
 
     #[test]
     fn compaction_filter_factory_test() {
-        let path = "_rust_rocksdb_filter_factory_test";
+        let tempdir = tempfile::Builder::new()
+            .prefix("_rust_rocksdb_filter_factory_test")
+            .tempdir()
+            .expect("Failed to create temporary path for the _rust_rocksdb_filter_factory_test.");
+        let path = tempdir.path();
         let mut opts = Options::default();
         opts.create_if_missing(true);
         opts.set_compaction_filter_factory(TestFactory(CString::new("TestFactory").unwrap()));
