@@ -26,6 +26,7 @@ use crate::{
     WaitForCompactOptions, WriteBatch, WriteOptions, DEFAULT_COLUMN_FAMILY_NAME,
 };
 
+use crate::column_family::ColumnFamilyTtl;
 use crate::ffi_util::CSlice;
 use libc::{self, c_char, c_int, c_uchar, c_void, size_t};
 use std::collections::BTreeMap;
@@ -42,6 +43,20 @@ use std::sync::Arc;
 use std::sync::RwLock;
 use std::time::Duration;
 
+/// A range of keys, `start_key` is included, but not `end_key`.
+///
+/// You should make sure `end_key` is not less than `start_key`.
+pub struct Range<'a> {
+    start_key: &'a [u8],
+    end_key: &'a [u8],
+}
+
+impl<'a> Range<'a> {
+    pub fn new(start_key: &'a [u8], end_key: &'a [u8]) -> Range<'a> {
+        Range { start_key, end_key }
+    }
+}
+
 /// Marker trait to specify single or multi threaded column family alternations for
 /// [`DBWithThreadMode<T>`]
 ///
@@ -196,15 +211,15 @@ pub trait DBAccess {
 
 impl<T: ThreadMode, D: DBInner> DBAccess for DBCommon<T, D> {
     unsafe fn create_snapshot(&self) -> *const ffi::rocksdb_snapshot_t {
-        ffi::rocksdb_create_snapshot(self.inner.inner())
+        unsafe { ffi::rocksdb_create_snapshot(self.inner.inner()) }
     }
 
     unsafe fn release_snapshot(&self, snapshot: *const ffi::rocksdb_snapshot_t) {
-        ffi::rocksdb_release_snapshot(self.inner.inner(), snapshot);
+        unsafe { ffi::rocksdb_release_snapshot(self.inner.inner(), snapshot) };
     }
 
     unsafe fn create_iterator(&self, readopts: &ReadOptions) -> *mut ffi::rocksdb_iterator_t {
-        ffi::rocksdb_create_iterator(self.inner.inner(), readopts.inner)
+        unsafe { ffi::rocksdb_create_iterator(self.inner.inner(), readopts.inner) }
     }
 
     unsafe fn create_iterator_cf(
@@ -212,7 +227,7 @@ impl<T: ThreadMode, D: DBInner> DBAccess for DBCommon<T, D> {
         cf_handle: *mut ffi::rocksdb_column_family_handle_t,
         readopts: &ReadOptions,
     ) -> *mut ffi::rocksdb_iterator_t {
-        ffi::rocksdb_create_iterator_cf(self.inner.inner(), readopts.inner, cf_handle)
+        unsafe { ffi::rocksdb_create_iterator_cf(self.inner.inner(), readopts.inner, cf_handle) }
     }
 
     fn get_opt<K: AsRef<[u8]>>(
@@ -377,6 +392,9 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
     }
 
     /// Opens the database with a Time to Live compaction filter.
+    ///
+    /// This applies the given `ttl` to all column families created without an explicit TTL.
+    /// See [`DB::open_cf_descriptors_with_ttl`] for more control over individual column family TTLs.
     pub fn open_with_ttl<P: AsRef<Path>>(
         opts: &Options,
         path: P,
@@ -408,6 +426,17 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
 
     /// Opens a database with the given database with a Time to Live compaction filter and
     /// column family descriptors.
+    ///
+    /// Applies the provided `ttl` as the default TTL for all column families.
+    /// Column families will inherit this TTL by default, unless their descriptor explicitly
+    /// sets a different TTL using [`ColumnFamilyTtl::Duration`] or opts out using [`ColumnFamilyTtl::Disabled`].
+    ///
+    /// *NOTE*: The `default` column family is opened with `Options::default()` unless
+    /// explicitly configured within the `cfs` iterator.
+    /// To customize the `default` column family's options, include a `ColumnFamilyDescriptor`
+    /// with the name "default" in the `cfs` iterator.
+    ///
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_descriptors_with_ttl<P, I>(
         opts: &Options,
         path: P,
@@ -454,6 +483,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
     }
 
     /// Opens a database for read only with the given database options and column family names.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_for_read_only<P, I, N>(
         opts: &Options,
         path: P,
@@ -480,6 +511,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
     }
 
     /// Opens a database for read only with the given database options and column family names.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_with_opts_for_read_only<P, I, N>(
         db_opts: &Options,
         path: P,
@@ -507,6 +540,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
 
     /// Opens a database for ready only with the given database options and
     /// column family descriptors.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_descriptors_read_only<P, I>(
         opts: &Options,
         path: P,
@@ -528,6 +563,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
     }
 
     /// Opens the database as a secondary with the given database options and column family names.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_as_secondary<P, I, N>(
         opts: &Options,
         primary_path: P,
@@ -555,6 +592,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
 
     /// Opens the database as a secondary with the given database options and
     /// column family descriptors.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_descriptors_as_secondary<P, I>(
         opts: &Options,
         path: P,
@@ -576,6 +615,8 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
     }
 
     /// Opens a database with the given database options and column family descriptors.
+    /// *NOTE*: `default` column family is opened with `Options::default()`.
+    /// If you want to open `default` cf with different options, set them explicitly in `cfs`.
     pub fn open_cf_descriptors<P, I>(opts: &Options, path: P, cfs: I) -> Result<Self, Error>
     where
         P: AsRef<Path>,
@@ -620,6 +661,7 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
                 cfs_v.push(ColumnFamilyDescriptor {
                     name: String::from(DEFAULT_COLUMN_FAMILY_NAME),
                     options: Options::default(),
+                    ttl: ColumnFamilyTtl::SameAsDb,
                 });
             }
             // We need to store our CStrings in an intermediate vector
@@ -636,7 +678,7 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
 
             let cfopts: Vec<_> = cfs_v
                 .iter()
-                .map(|cf| cf.options.inner as *const _)
+                .map(|cf| cf.options.inner.cast_const())
                 .collect();
 
             db = Self::open_cf_raw(
@@ -750,7 +792,15 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
                     ))
                 }
                 AccessType::WithTTL { ttl } => {
-                    let ttls_v = vec![ttl.as_secs() as c_int; cfs_v.len()];
+                    let ttls: Vec<_> = cfs_v
+                        .iter()
+                        .map(|cf| match cf.ttl {
+                            ColumnFamilyTtl::Disabled => i32::MAX,
+                            ColumnFamilyTtl::Duration(duration) => duration.as_secs() as i32,
+                            ColumnFamilyTtl::SameAsDb => ttl.as_secs() as i32,
+                        })
+                        .collect();
+
                     ffi_try!(ffi::rocksdb_open_column_families_with_ttl(
                         opts.inner,
                         cpath.as_ptr(),
@@ -758,7 +808,7 @@ impl<T: ThreadMode> DBWithThreadMode<T> {
                         cfnames.as_ptr(),
                         cfopts.as_ptr(),
                         cfhandles.as_mut_ptr(),
-                        ttls_v.as_ptr(),
+                        ttls.as_ptr(),
                     ))
                 }
             }
@@ -1090,7 +1140,10 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
     {
         let (keys, keys_sizes): (Vec<Box<[u8]>>, Vec<_>) = keys
             .into_iter()
-            .map(|k| (Box::from(k.as_ref()), k.as_ref().len()))
+            .map(|k| {
+                let k = k.as_ref();
+                (Box::from(k), k.len())
+            })
             .unzip();
         let ptr_keys: Vec<_> = keys.iter().map(|k| k.as_ptr() as *const c_char).collect();
 
@@ -1139,7 +1192,10 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
     {
         let (cfs_and_keys, keys_sizes): (Vec<(_, Box<[u8]>)>, Vec<_>) = keys
             .into_iter()
-            .map(|(cf, key)| ((cf, Box::from(key.as_ref())), key.as_ref().len()))
+            .map(|(cf, key)| {
+                let key = key.as_ref();
+                ((cf, Box::from(key)), key.len())
+            })
             .unzip();
         let ptr_keys: Vec<_> = cfs_and_keys
             .iter()
@@ -1147,7 +1203,7 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
             .collect();
         let ptr_cfs: Vec<_> = cfs_and_keys
             .iter()
-            .map(|(c, _)| c.inner() as *const _)
+            .map(|(c, _)| c.inner().cast_const())
             .collect();
 
         let mut values = vec![ptr::null_mut(); ptr_keys.len()];
@@ -1225,7 +1281,7 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
             );
             pinned_values
                 .into_iter()
-                .zip(errors.into_iter())
+                .zip(errors)
                 .map(|(v, e)| {
                     if e.is_null() {
                         if v.is_null() {
@@ -1530,6 +1586,80 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         }
     }
 
+    /// Set the database entry for "key" to "value" with WriteOptions.
+    /// If "key" already exists, it will coexist with previous entry.
+    /// `Get` with a timestamp ts specified in ReadOptions will return
+    /// the most recent key/value whose timestamp is smaller than or equal to ts.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn put_with_ts_opt<K, V, S>(
+        &self,
+        key: K,
+        ts: S,
+        value: V,
+        writeopts: &WriteOptions,
+    ) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        V: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        let key = key.as_ref();
+        let value = value.as_ref();
+        let ts = ts.as_ref();
+        unsafe {
+            ffi_try!(ffi::rocksdb_put_with_ts(
+                self.inner.inner(),
+                writeopts.inner,
+                key.as_ptr() as *const c_char,
+                key.len() as size_t,
+                ts.as_ptr() as *const c_char,
+                ts.len() as size_t,
+                value.as_ptr() as *const c_char,
+                value.len() as size_t,
+            ));
+            Ok(())
+        }
+    }
+
+    /// Put with timestamp in a specific column family with WriteOptions.
+    /// If "key" already exists, it will coexist with previous entry.
+    /// `Get` with a timestamp ts specified in ReadOptions will return
+    /// the most recent key/value whose timestamp is smaller than or equal to ts.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn put_cf_with_ts_opt<K, V, S>(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        key: K,
+        ts: S,
+        value: V,
+        writeopts: &WriteOptions,
+    ) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        V: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        let key = key.as_ref();
+        let value = value.as_ref();
+        let ts = ts.as_ref();
+        unsafe {
+            ffi_try!(ffi::rocksdb_put_cf_with_ts(
+                self.inner.inner(),
+                writeopts.inner,
+                cf.inner(),
+                key.as_ptr() as *const c_char,
+                key.len() as size_t,
+                ts.as_ptr() as *const c_char,
+                ts.len() as size_t,
+                value.as_ptr() as *const c_char,
+                value.len() as size_t,
+            ));
+            Ok(())
+        }
+    }
+
     pub fn merge_opt<K, V>(&self, key: K, value: V, writeopts: &WriteOptions) -> Result<(), Error>
     where
         K: AsRef<[u8]>,
@@ -1617,6 +1747,64 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         }
     }
 
+    /// Remove the database entry (if any) for "key" with WriteOptions.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn delete_with_ts_opt<K, S>(
+        &self,
+        key: K,
+        ts: S,
+        writeopts: &WriteOptions,
+    ) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        let key = key.as_ref();
+        let ts = ts.as_ref();
+        unsafe {
+            ffi_try!(ffi::rocksdb_delete_with_ts(
+                self.inner.inner(),
+                writeopts.inner,
+                key.as_ptr() as *const c_char,
+                key.len() as size_t,
+                ts.as_ptr() as *const c_char,
+                ts.len() as size_t,
+            ));
+            Ok(())
+        }
+    }
+
+    /// Delete with timestamp in a specific column family with WriteOptions.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn delete_cf_with_ts_opt<K, S>(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        key: K,
+        ts: S,
+        writeopts: &WriteOptions,
+    ) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        let key = key.as_ref();
+        let ts = ts.as_ref();
+        unsafe {
+            ffi_try!(ffi::rocksdb_delete_cf_with_ts(
+                self.inner.inner(),
+                writeopts.inner,
+                cf.inner(),
+                key.as_ptr() as *const c_char,
+                key.len() as size_t,
+                ts.as_ptr() as *const c_char,
+                ts.len() as size_t,
+            ));
+            Ok(())
+        }
+    }
+
     pub fn put<K, V>(&self, key: K, value: V) -> Result<(), Error>
     where
         K: AsRef<[u8]>,
@@ -1633,6 +1821,53 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         self.put_cf_opt(cf, key.as_ref(), value.as_ref(), &WriteOptions::default())
     }
 
+    /// Set the database entry for "key" to "value".
+    /// If "key" already exists, it will coexist with previous entry.
+    /// `Get` with a timestamp ts specified in ReadOptions will return
+    /// the most recent key/value whose timestamp is smaller than or equal to ts.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn put_with_ts<K, V, S>(&self, key: K, ts: S, value: V) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        V: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        self.put_with_ts_opt(
+            key.as_ref(),
+            ts.as_ref(),
+            value.as_ref(),
+            &WriteOptions::default(),
+        )
+    }
+
+    /// Put with timestamp in a specific column family.
+    /// If "key" already exists, it will coexist with previous entry.
+    /// `Get` with a timestamp ts specified in ReadOptions will return
+    /// the most recent key/value whose timestamp is smaller than or equal to ts.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn put_cf_with_ts<K, V, S>(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        key: K,
+        ts: S,
+        value: V,
+    ) -> Result<(), Error>
+    where
+        K: AsRef<[u8]>,
+        V: AsRef<[u8]>,
+        S: AsRef<[u8]>,
+    {
+        self.put_cf_with_ts_opt(
+            cf,
+            key.as_ref(),
+            ts.as_ref(),
+            value.as_ref(),
+            &WriteOptions::default(),
+        )
+    }
+
     pub fn merge<K, V>(&self, key: K, value: V) -> Result<(), Error>
     where
         K: AsRef<[u8]>,
@@ -1661,6 +1896,29 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         self.delete_cf_opt(cf, key.as_ref(), &WriteOptions::default())
     }
 
+    /// Remove the database entry (if any) for "key".
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn delete_with_ts<K: AsRef<[u8]>, S: AsRef<[u8]>>(
+        &self,
+        key: K,
+        ts: S,
+    ) -> Result<(), Error> {
+        self.delete_with_ts_opt(key.as_ref(), ts.as_ref(), &WriteOptions::default())
+    }
+
+    /// Delete with timestamp in a specific column family.
+    /// Takes an additional argument `ts` as the timestamp.
+    /// Note: the DB must be opened with user defined timestamp enabled.
+    pub fn delete_cf_with_ts<K: AsRef<[u8]>, S: AsRef<[u8]>>(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        key: K,
+        ts: S,
+    ) -> Result<(), Error> {
+        self.delete_cf_with_ts_opt(cf, key.as_ref(), ts.as_ref(), &WriteOptions::default())
+    }
+
     /// Runs a manual compaction on the Range of keys given. This is not likely to be needed for typical usage.
     pub fn compact_range<S: AsRef<[u8]>, E: AsRef<[u8]>>(&self, start: Option<S>, end: Option<E>) {
         unsafe {
@@ -1910,6 +2168,80 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         unsafe { ffi::rocksdb_get_latest_sequence_number(self.inner.inner()) }
     }
 
+    /// Return the approximate file system space used by keys in each ranges.
+    ///
+    /// Note that the returned sizes measure file system space usage, so
+    /// if the user data compresses by a factor of ten, the returned
+    /// sizes will be one-tenth the size of the corresponding user data size.
+    ///
+    /// Due to lack of abi, only data flushed to disk is taken into account.
+    pub fn get_approximate_sizes(&self, ranges: &[Range]) -> Vec<u64> {
+        self.get_approximate_sizes_cfopt(None::<&ColumnFamily>, ranges)
+    }
+
+    pub fn get_approximate_sizes_cf(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        ranges: &[Range],
+    ) -> Vec<u64> {
+        self.get_approximate_sizes_cfopt(Some(cf), ranges)
+    }
+
+    fn get_approximate_sizes_cfopt(
+        &self,
+        cf: Option<&impl AsColumnFamilyRef>,
+        ranges: &[Range],
+    ) -> Vec<u64> {
+        let start_keys: Vec<*const c_char> = ranges
+            .iter()
+            .map(|x| x.start_key.as_ptr() as *const c_char)
+            .collect();
+        let start_key_lens: Vec<_> = ranges.iter().map(|x| x.start_key.len()).collect();
+        let end_keys: Vec<*const c_char> = ranges
+            .iter()
+            .map(|x| x.end_key.as_ptr() as *const c_char)
+            .collect();
+        let end_key_lens: Vec<_> = ranges.iter().map(|x| x.end_key.len()).collect();
+        let mut sizes: Vec<u64> = vec![0; ranges.len()];
+        let (n, start_key_ptr, start_key_len_ptr, end_key_ptr, end_key_len_ptr, size_ptr) = (
+            ranges.len() as i32,
+            start_keys.as_ptr(),
+            start_key_lens.as_ptr(),
+            end_keys.as_ptr(),
+            end_key_lens.as_ptr(),
+            sizes.as_mut_ptr(),
+        );
+        let mut err: *mut c_char = ptr::null_mut();
+        match cf {
+            None => unsafe {
+                ffi::rocksdb_approximate_sizes(
+                    self.inner.inner(),
+                    n,
+                    start_key_ptr,
+                    start_key_len_ptr,
+                    end_key_ptr,
+                    end_key_len_ptr,
+                    size_ptr,
+                    &mut err,
+                );
+            },
+            Some(cf) => unsafe {
+                ffi::rocksdb_approximate_sizes_cf(
+                    self.inner.inner(),
+                    cf.inner(),
+                    n,
+                    start_key_ptr,
+                    start_key_len_ptr,
+                    end_key_ptr,
+                    end_key_len_ptr,
+                    size_ptr,
+                    &mut err,
+                );
+            },
+        }
+        sizes
+    }
+
     /// Iterate over batches of write operations since a given sequence.
     ///
     /// Produce an iterator that will provide the batches of write operations
@@ -2000,7 +2332,7 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
                 self.inner.inner(),
                 cpaths.as_ptr(),
                 paths_v.len(),
-                opts.inner as *const _
+                opts.inner.cast_const()
             ));
             Ok(())
         }
@@ -2019,7 +2351,7 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
                 cf.inner(),
                 cpaths.as_ptr(),
                 paths_v.len(),
-                opts.inner as *const _
+                opts.inner.cast_const()
             ));
             Ok(())
         }
@@ -2184,6 +2516,61 @@ impl<T: ThreadMode, D: DBInner> DBCommon<T, D> {
         drop(cf);
         Ok(())
     }
+
+    /// Increase the full_history_ts of column family. The new ts_low value should
+    /// be newer than current full_history_ts value.
+    /// If another thread updates full_history_ts_low concurrently to a higher
+    /// timestamp than the requested ts_low, a try again error will be returned.
+    pub fn increase_full_history_ts_low<S: AsRef<[u8]>>(
+        &self,
+        cf: &impl AsColumnFamilyRef,
+        ts: S,
+    ) -> Result<(), Error> {
+        let ts = ts.as_ref();
+        unsafe {
+            ffi_try!(ffi::rocksdb_increase_full_history_ts_low(
+                self.inner.inner(),
+                cf.inner(),
+                ts.as_ptr() as *const c_char,
+                ts.len() as size_t,
+            ));
+            Ok(())
+        }
+    }
+
+    /// Get current full_history_ts value.
+    pub fn get_full_history_ts_low(&self, cf: &impl AsColumnFamilyRef) -> Result<Vec<u8>, Error> {
+        unsafe {
+            let mut ts_lowlen = 0;
+            let ts = ffi_try!(ffi::rocksdb_get_full_history_ts_low(
+                self.inner.inner(),
+                cf.inner(),
+                &mut ts_lowlen,
+            ));
+
+            if ts.is_null() {
+                Err(Error::new("Could not get full_history_ts_low".to_owned()))
+            } else {
+                let mut vec = vec![0; ts_lowlen];
+                ptr::copy_nonoverlapping(ts as *mut u8, vec.as_mut_ptr(), ts_lowlen);
+                ffi::rocksdb_free(ts as *mut c_void);
+                Ok(vec)
+            }
+        }
+    }
+
+    /// Returns the DB identity. This is typically ASCII bytes, but that is not guaranteed.
+    pub fn get_db_identity(&self) -> Result<Vec<u8>, Error> {
+        unsafe {
+            let mut length: usize = 0;
+            let identity_ptr = ffi::rocksdb_get_db_identity(self.inner.inner(), &mut length);
+            let identity_vec = raw_data(identity_ptr, length);
+            ffi::rocksdb_free(identity_ptr as *mut c_void);
+            // In RocksDB: get_db_identity copies a std::string so it should not fail, but
+            // the API allows it to be overridden, so it might
+            identity_vec.ok_or_else(|| Error::new("get_db_identity returned NULL".to_string()))
+        }
+    }
 }
 
 impl<I: DBInner> DBCommon<SingleThreaded, I> {
@@ -2214,8 +2601,11 @@ impl<I: DBInner> DBCommon<SingleThreaded, I> {
 impl<I: DBInner> DBCommon<MultiThreaded, I> {
     /// Creates column family with given name and options
     pub fn create_cf<N: AsRef<str>>(&self, name: N, opts: &Options) -> Result<(), Error> {
+        // Note that we acquire the cfs lock before inserting: otherwise we might race
+        // another caller who observed the handle as missing.
+        let mut cfs = self.cfs.cfs.write().unwrap();
         let inner = self.create_inner_cf_handle(name.as_ref(), opts)?;
-        self.cfs.cfs.write().unwrap().insert(
+        cfs.insert(
             name.as_ref().to_string(),
             Arc::new(UnboundColumnFamily { inner }),
         );
@@ -2312,8 +2702,8 @@ pub(crate) fn convert_values(
 ) -> Vec<Result<Option<Vec<u8>>, Error>> {
     values
         .into_iter()
-        .zip(values_sizes.into_iter())
-        .zip(errors.into_iter())
+        .zip(values_sizes)
+        .zip(errors)
         .map(|((v, s), e)| {
             if e.is_null() {
                 let value = unsafe { crate::ffi_util::raw_data(v, s) };
