@@ -15,20 +15,21 @@
 mod util;
 
 use std::convert::TryInto;
-use std::{mem, sync::Arc, thread, time::Duration};
+use std::sync::atomic::{AtomicUsize, Ordering};
+use std::{sync::Arc, thread, time::Duration};
 
 use pretty_assertions::assert_eq;
 
 use rocksdb::statistics::{Histogram, StatsLevel, Ticker};
 use rocksdb::{
     perf::get_memory_usage_stats, BlockBasedOptions, BottommostLevelCompaction, Cache,
-    ColumnFamilyDescriptor, CompactOptions, CuckooTableOptions, DBAccess, DBCompactionStyle,
-    DBWithThreadMode, Env, Error, ErrorKind, FifoCompactOptions, IteratorMode, MultiThreaded,
-    Options, PerfContext, PerfMetric, ReadOptions, SingleThreaded, SliceTransform, Snapshot,
-    UniversalCompactOptions, UniversalCompactionStopStyle, WaitForCompactOptions, WriteBatch, DB,
-    DEFAULT_COLUMN_FAMILY_NAME,
+    ColumnFamilyDescriptor, ColumnFamilyTtl, CompactOptions, CuckooTableOptions, DBAccess,
+    DBCompactionStyle, DBWithThreadMode, Env, Error, ErrorKind, FifoCompactOptions, IteratorMode,
+    MultiThreaded, Options, PerfContext, PerfMetric, ReadOptions, SingleThreaded, SliceTransform,
+    Snapshot, UniversalCompactOptions, UniversalCompactionStopStyle, WaitForCompactOptions,
+    WriteBatch, DB, DEFAULT_COLUMN_FAMILY_NAME,
 };
-use util::{assert_iter, pair, DBPath};
+use util::{assert_iter, pair, DBPath, U64Comparator, U64Timestamp};
 
 #[test]
 fn external() {
@@ -263,14 +264,14 @@ fn snapshot_test() {
 }
 
 #[derive(Clone)]
-struct SnapshotWrapper {
-    snapshot: Arc<Snapshot<'static>>,
+struct SnapshotWrapper<'db> {
+    snapshot: Arc<Snapshot<'db>>,
 }
 
-impl SnapshotWrapper {
-    fn new(db: &DB) -> Self {
+impl<'db> SnapshotWrapper<'db> {
+    fn new(db: &'db DB) -> Self {
         Self {
-            snapshot: Arc::new(unsafe { mem::transmute(db.snapshot()) }),
+            snapshot: Arc::new(db.snapshot()),
         }
     }
 
@@ -290,13 +291,14 @@ fn sync_snapshot_test() {
     assert!(db.put(b"k1", b"v1").is_ok());
     assert!(db.put(b"k2", b"v2").is_ok());
 
-    let wrapper = SnapshotWrapper::new(&db);
-    let wrapper_1 = wrapper.clone();
-    let handler_1 = thread::spawn(move || wrapper_1.check("k1", b"v1"));
-    let handler_2 = thread::spawn(move || wrapper.check("k2", b"v2"));
-
-    assert!(handler_1.join().unwrap());
-    assert!(handler_2.join().unwrap());
+    let wrapper_1 = SnapshotWrapper::new(&db);
+    let wrapper_2 = wrapper_1.clone();
+    thread::scope(|s| {
+        let handler_1 = s.spawn(move || wrapper_1.check("k1", b"v1"));
+        let handler_2 = s.spawn(move || wrapper_2.check("k2", b"v2"));
+        assert!(handler_1.join().unwrap());
+        assert!(handler_2.join().unwrap());
+    });
 }
 
 #[test]
@@ -452,10 +454,10 @@ struct OperationCounts {
 }
 
 impl rocksdb::WriteBatchIterator for OperationCounts {
-    fn put(&mut self, _key: Box<[u8]>, _value: Box<[u8]>) {
+    fn put(&mut self, _key: &[u8], _value: &[u8]) {
         self.puts += 1;
     }
-    fn delete(&mut self, _key: Box<[u8]>) {
+    fn delete(&mut self, _key: &[u8]) {
         self.deletes += 1;
     }
 }
@@ -534,29 +536,31 @@ fn test_get_updates_since_multiple_batches() {
 
 #[test]
 fn test_get_updates_since_one_batch() {
-    let path = DBPath::new("_rust_rocksdb_test_get_updates_since_one_batch");
-    let db = DB::open_default(&path).unwrap();
-    db.put(b"key2", b"value2").unwrap();
-    // some puts and deletes in a single batch,
-    // verify 1 put and 1 delete were done
-    let seq1 = db.latest_sequence_number();
-    assert_eq!(seq1, 1);
-    let mut batch = WriteBatch::default();
-    batch.put(b"key1", b"value1");
-    batch.delete(b"key2");
-    db.write(batch).unwrap();
-    assert_eq!(db.latest_sequence_number(), 3);
-    let mut iter = db.get_updates_since(seq1).unwrap();
-    let mut counts = OperationCounts {
-        puts: 0,
-        deletes: 0,
-    };
-    let (seq, batch) = iter.next().unwrap().unwrap();
-    assert_eq!(seq, 2);
-    batch.iterate(&mut counts);
-    assert!(iter.next().is_none());
-    assert_eq!(counts.puts, 1);
-    assert_eq!(counts.deletes, 1);
+    let batches = [WriteBatch::default(), WriteBatch::with_capacity_bytes(13)];
+    for mut batch in batches {
+        let path = DBPath::new("_rust_rocksdb_test_get_updates_since_one_batch");
+        let db = DB::open_default(&path).unwrap();
+        db.put(b"key2", b"value2").unwrap();
+        // some puts and deletes in a single batch,
+        // verify 1 put and 1 delete were done
+        let seq1 = db.latest_sequence_number();
+        assert_eq!(seq1, 1);
+        batch.put(b"key1", b"value1");
+        batch.delete(b"key2");
+        db.write(batch).unwrap();
+        assert_eq!(db.latest_sequence_number(), 3);
+        let mut iter = db.get_updates_since(seq1).unwrap();
+        let mut counts = OperationCounts {
+            puts: 0,
+            deletes: 0,
+        };
+        let (seq, batch) = iter.next().unwrap().unwrap();
+        assert_eq!(seq, 2);
+        batch.iterate(&mut counts);
+        assert!(iter.next().is_none());
+        assert_eq!(counts.puts, 1);
+        assert_eq!(counts.deletes, 1);
+    }
 }
 
 #[test]
@@ -691,6 +695,46 @@ fn test_open_with_ttl() {
     assert!(db.get(b"key1").unwrap().is_none());
 }
 
+#[test]
+fn test_ttl_mix() {
+    let path = DBPath::new("_rust_rocksdb_test_open_with_ttl_mix");
+
+    let mut opts = Options::default();
+    opts.create_if_missing(true);
+    opts.create_missing_column_families(true);
+
+    let cf1 = ColumnFamilyDescriptor::new_with_ttl(
+        "ttl_1",
+        Options::default(),
+        ColumnFamilyTtl::Duration(Duration::from_secs(1)),
+    );
+    let no_ttl = ColumnFamilyDescriptor::new_with_ttl(
+        "no_ttl",
+        Options::default(),
+        ColumnFamilyTtl::Disabled,
+    );
+
+    let db = DB::open_cf_descriptors_with_ttl(&opts, &path, [cf1, no_ttl], Duration::from_secs(1))
+        .unwrap();
+    db.put(b"key1", b"value1").unwrap();
+
+    let cf1 = db.cf_handle("ttl_1").unwrap();
+    let no_ttl = db.cf_handle("no_ttl").unwrap();
+
+    db.put_cf(&cf1, b"key2", b"value2").unwrap();
+    db.put_cf(&no_ttl, b"key3", b"value3").unwrap();
+
+    thread::sleep(Duration::from_secs(2));
+    // Trigger a manual compaction, this will check the TTL filter
+    // in the database and drop all expired entries.
+    db.compact_range(None::<&[u8]>, None::<&[u8]>);
+    db.compact_range_cf(&cf1, None::<&[u8]>, None::<&[u8]>);
+    db.compact_range_cf(&no_ttl, None::<&[u8]>, None::<&[u8]>);
+    assert!(db.get(b"key1").unwrap().is_none());
+    assert!(db.get_cf(&cf1, b"key2").unwrap().is_none());
+    assert!(db.get_cf(&no_ttl, b"key3").unwrap().is_some());
+}
+
 #[test]
 fn test_open_cf_with_ttl() {
     let path = DBPath::new("_rust_rocksdb_test_open_cf_with_ttl");
@@ -742,7 +786,7 @@ fn test_open_with_multiple_refs_as_single_threaded() {
 
 #[test]
 fn test_open_utf8_path() {
-    let path = DBPath::new("_rust_rocksdb_utf8_path_temporärer_Ordner");
+    let path = DBPath::new("_rust_rocksdb_utf8_path_temporärer_Order");
 
     {
         let db = DB::open_default(&path).unwrap();
@@ -952,134 +996,140 @@ fn prefix_extract_and_iterate_test() {
 
 #[test]
 fn get_with_cache_and_bulkload_test() {
-    let path = DBPath::new("_rust_rocksdb_get_with_cache_and_bulkload_test");
-    let log_path = DBPath::new("_rust_rocksdb_log_path_test");
+    let batches = [
+        WriteBatch::default(),
+        WriteBatch::with_capacity_bytes(13),
+        WriteBatch::with_capacity_bytes(100_000),
+    ];
+    for mut batch in batches {
+        let path = DBPath::new("_rust_rocksdb_get_with_cache_and_bulkload_test");
+        let log_path = DBPath::new("_rust_rocksdb_log_path_test");
 
-    // create options
-    let mut opts = Options::default();
-    opts.create_if_missing(true);
-    opts.create_missing_column_families(true);
-    opts.set_wal_bytes_per_sync(8 << 10); // 8KB
-    opts.set_writable_file_max_buffer_size(512 << 10); // 512KB
-    opts.set_enable_write_thread_adaptive_yield(true);
-    opts.set_unordered_write(true);
-    opts.set_max_subcompactions(2);
-    opts.set_max_background_jobs(4);
-    opts.set_use_adaptive_mutex(true);
-    opts.set_db_log_dir(&log_path);
-    opts.set_memtable_whole_key_filtering(true);
-    opts.set_dump_malloc_stats(true);
-    opts.set_level_compaction_dynamic_level_bytes(false);
-
-    // trigger all sst files in L1/2 instead of L0
-    opts.set_max_bytes_for_level_base(64 << 10); // 64KB
-
-    {
-        // set block based table and cache
-        let cache = Cache::new_lru_cache(512 << 10);
-        assert_eq!(cache.get_usage(), 0);
-        let mut block_based_opts = BlockBasedOptions::default();
-        block_based_opts.set_block_cache(&cache);
-        block_based_opts.set_cache_index_and_filter_blocks(true);
-        opts.set_block_based_table_factory(&block_based_opts);
-
-        // open db
-        let db = DB::open(&opts, &path).unwrap();
-
-        // write a lot
-        let mut batch = WriteBatch::default();
-        for i in 0..10_000 {
-            batch.put(format!("{i:0>4}").as_bytes(), b"v");
-        }
-        assert!(db.write(batch).is_ok());
-
-        // flush memory table to sst and manual compaction
-        assert!(db.flush().is_ok());
-        db.compact_range(Some(format!("{:0>4}", 0).as_bytes()), None::<Vec<u8>>);
-
-        // get -> trigger caching
-        let _ = db.get(b"1");
-        assert!(cache.get_usage() > 0);
-
-        // get approximated memory usage
-        let mem_usage = get_memory_usage_stats(Some(&[&db]), None).unwrap();
-        assert!(mem_usage.mem_table_total > 0);
-
-        // get approximated cache usage
-        let mem_usage = get_memory_usage_stats(None, Some(&[&cache])).unwrap();
-        assert!(mem_usage.cache_total > 0);
-    }
-
-    // bulk loading
-    {
-        // open db
-        let db = DB::open(&opts, &path).unwrap();
-
-        // try to get key
-        let iter = db.iterator(IteratorMode::Start);
-        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
-            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
-        }
-
-        // check live files (sst files meta)
-        let livefiles = db.live_files().unwrap();
-        assert_eq!(livefiles.len(), 1);
-        livefiles.iter().for_each(|f| {
-            assert_eq!(f.level, 2);
-            assert_eq!(f.column_family_name, "default");
-            assert!(!f.name.is_empty());
-            assert_eq!(
-                f.start_key.as_ref().unwrap().as_slice(),
-                format!("{:0>4}", 0).as_bytes()
-            );
-            assert_eq!(
-                f.end_key.as_ref().unwrap().as_slice(),
-                format!("{:0>4}", 9999).as_bytes()
-            );
-            assert_eq!(f.num_entries, 10000);
-            assert_eq!(f.num_deletions, 0);
-        });
-
-        // delete sst file in range (except L0)
-        assert!(db
-            .delete_file_in_range(
-                format!("{:0>4}", 0).as_bytes(),
-                format!("{:0>4}", 9999).as_bytes()
-            )
-            .is_ok());
-        let livefiles = db.live_files().unwrap();
-        assert_eq!(livefiles.len(), 0);
-
-        // try to get a deleted key
-        assert!(db.get(format!("{:0>4}", 123).as_bytes()).unwrap().is_none());
-    }
-
-    // raise error when db exists
-    {
-        opts.set_error_if_exists(true);
-        assert!(DB::open(&opts, &path).is_err());
-    }
-
-    // disable all threads
-    {
-        // create new options
+        // create options
         let mut opts = Options::default();
-        opts.set_max_background_jobs(0);
-        opts.set_stats_dump_period_sec(0);
-        opts.set_stats_persist_period_sec(0);
+        opts.create_if_missing(true);
+        opts.create_missing_column_families(true);
+        opts.set_wal_bytes_per_sync(8 << 10); // 8KB
+        opts.set_writable_file_max_buffer_size(512 << 10); // 512KB
+        opts.set_enable_write_thread_adaptive_yield(true);
+        opts.set_unordered_write(true);
+        opts.set_max_subcompactions(2);
+        opts.set_max_background_jobs(4);
+        opts.set_use_adaptive_mutex(true);
+        opts.set_db_log_dir(&log_path);
+        opts.set_memtable_whole_key_filtering(true);
+        opts.set_dump_malloc_stats(true);
+        opts.set_level_compaction_dynamic_level_bytes(false);
 
-        // test Env::Default()->SetBackgroundThreads(0, Env::Priority::BOTTOM);
-        let mut env = Env::new().unwrap();
-        env.set_bottom_priority_background_threads(0);
-        opts.set_env(&env);
+        // trigger all sst files in L1/2 instead of L0
+        opts.set_max_bytes_for_level_base(64 << 10); // 64KB
 
-        // open db
-        let db = DB::open(&opts, &path).unwrap();
+        {
+            // set block based table and cache
+            let cache = Cache::new_lru_cache(512 << 10);
+            assert_eq!(cache.get_usage(), 0);
+            let mut block_based_opts = BlockBasedOptions::default();
+            block_based_opts.set_block_cache(&cache);
+            block_based_opts.set_cache_index_and_filter_blocks(true);
+            opts.set_block_based_table_factory(&block_based_opts);
 
-        // try to get key
-        let iter = db.iterator(IteratorMode::Start);
-        for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
-            assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
+            // open db
+            let db = DB::open(&opts, &path).unwrap();
+
+            // write a lot
+            for i in 0..10_000 {
+                batch.put(format!("{i:0>4}").as_bytes(), b"v");
+            }
+            assert!(db.write(batch).is_ok());
+
+            // flush memory table to sst and manual compaction
+            assert!(db.flush().is_ok());
+            db.compact_range(Some(format!("{:0>4}", 0).as_bytes()), None::<Vec<u8>>);
+
+            // get -> trigger caching
+            let _ = db.get(b"1");
+            assert!(cache.get_usage() > 0);
+
+            // get approximated memory usage
+            let mem_usage = get_memory_usage_stats(Some(&[&db]), None).unwrap();
+            assert!(mem_usage.mem_table_total > 0);
+
+            // get approximated cache usage
+            let mem_usage = get_memory_usage_stats(None, Some(&[&cache])).unwrap();
+            assert!(mem_usage.cache_total > 0);
+        }
+
+        // bulk loading
+        {
+            // open db
+            let db = DB::open(&opts, &path).unwrap();
+
+            // try to get key
+            let iter = db.iterator(IteratorMode::Start);
+            for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+                assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
+            }
+
+            // check live files (sst files meta)
+            let livefiles = db.live_files().unwrap();
+            assert_eq!(livefiles.len(), 1);
+            livefiles.iter().for_each(|f| {
+                assert_eq!(f.level, 2);
+                assert_eq!(f.column_family_name, "default");
+                assert!(!f.name.is_empty());
+                assert_eq!(
+                    f.start_key.as_ref().unwrap().as_slice(),
+                    format!("{:0>4}", 0).as_bytes()
+                );
+                assert_eq!(
+                    f.end_key.as_ref().unwrap().as_slice(),
+                    format!("{:0>4}", 9999).as_bytes()
+                );
+                assert_eq!(f.num_entries, 10000);
+                assert_eq!(f.num_deletions, 0);
+            });
+
+            // delete sst file in range (except L0)
+            assert!(db
+                .delete_file_in_range(
+                    format!("{:0>4}", 0).as_bytes(),
+                    format!("{:0>4}", 9999).as_bytes()
+                )
+                .is_ok());
+            let livefiles = db.live_files().unwrap();
+            assert_eq!(livefiles.len(), 0);
+
+            // try to get a deleted key
+            assert!(db.get(format!("{:0>4}", 123).as_bytes()).unwrap().is_none());
+        }
+
+        // raise error when db exists
+        {
+            opts.set_error_if_exists(true);
+            assert!(DB::open(&opts, &path).is_err());
+        }
+
+        // disable all threads
+        {
+            // create new options
+            let mut opts = Options::default();
+            opts.set_max_background_jobs(0);
+            opts.set_stats_dump_period_sec(0);
+            opts.set_stats_persist_period_sec(0);
+
+            // test Env::Default()->SetBackgroundThreads(0, Env::Priority::BOTTOM);
+            let mut env = Env::new().unwrap();
+            env.set_bottom_priority_background_threads(0);
+            opts.set_env(&env);
+
+            // open db
+            let db = DB::open(&opts, &path).unwrap();
+
+            // try to get key
+            let iter = db.iterator(IteratorMode::Start);
+            for (expected, (k, _)) in iter.map(Result::unwrap).enumerate() {
+                assert_eq!(k.as_ref(), format!("{expected:0>4}").as_bytes());
+            }
         }
     }
 }
@@ -1525,6 +1575,34 @@ fn cuckoo() {
     }
 }
 
+#[derive(Default)]
+struct EvilAsRef {
+    toggle: AtomicUsize,
+}
+
+impl AsRef<[u8]> for EvilAsRef {
+    fn as_ref(&self) -> &[u8] {
+        if self.toggle.fetch_xor(1, Ordering::Relaxed) == 0 {
+            b""
+        } else {
+            b"lorem ipsum dolor sit amet"
+        }
+    }
+}
+
+#[test]
+fn evil_as_ref() {
+    let path = DBPath::new("_rust_rocksdb_evil_as_ref");
+    let db = DB::open_default(&path).unwrap();
+
+    let evil = EvilAsRef {
+        toggle: AtomicUsize::new(0),
+    };
+
+    let result = &db.multi_get([evil])[0];
+    assert!(result.as_ref().unwrap().is_none());
+}
+
 #[test]
 fn test_atomic_flush_cfs() {
     let n = DBPath::new("_rust_rocksdb_atomic_flush_cfs");
@@ -1569,3 +1647,74 @@ fn test_atomic_flush_cfs() {
         );
     }
 }
+
+#[test]
+fn test_full_history_ts_low() {
+    let path = DBPath::new("_rust_full_history_ts_low");
+    let _ = DB::destroy(&Options::default(), &path);
+
+    {
+        let mut opts = Options::default();
+        opts.create_if_missing(true);
+        opts.create_missing_column_families(true);
+
+        let mut cf_opts = Options::default();
+        cf_opts.set_comparator_with_ts(
+            U64Comparator::NAME,
+            U64Timestamp::SIZE,
+            Box::new(U64Comparator::compare),
+            Box::new(U64Comparator::compare_ts),
+            Box::new(U64Comparator::compare_without_ts),
+        );
+
+        let cfs = vec![("cf", cf_opts)];
+
+        let db = DB::open_cf_with_opts(&opts, &path, cfs).unwrap();
+        let cf = db.cf_handle("cf").unwrap();
+
+        let ts = U64Timestamp::new(1);
+        db.increase_full_history_ts_low(&cf, ts).unwrap();
+        let ret = U64Timestamp::from(db.get_full_history_ts_low(&cf).unwrap().as_slice());
+        assert_eq!(ts, ret);
+
+        let _ = DB::destroy(&Options::default(), &path);
+    }
+}
+
+#[test]
+fn test_get_approximate_sizes_cf() {
+    let path = DBPath::new("_rust_rocksdb_get_approximate_sizes_cf_test");
+    let _ = DB::destroy(&Options::default(), &path);
+
+    {
+        let mut opts = Options::default();
+        opts.create_if_missing(true);
+        opts.create_missing_column_families(true);
+
+        let cf_opts = Options::default();
+        let cfs = vec![("default", cf_opts)];
+
+        let db = DB::open_cf_with_opts(&opts, &path, cfs).unwrap();
+        let cf = db.cf_handle("default").unwrap();
+
+        // Insert some data
+        for i in 0..1000 {
+            let key = format!("key_{i:04}");
+            let value = format!("value_{i:04}");
+            db.put_cf(&cf, key.as_bytes(), value.as_bytes()).unwrap();
+        }
+
+        // Flush to ensure data is written to disk
+        db.flush_cf(&cf).unwrap();
+
+        // Get approximate sizes
+        let start_key = b"key_0000";
+        let end_key = b"key_0999";
+        let sizes = db.get_approximate_sizes_cf(&cf, &[rocksdb::Range::new(start_key, end_key)]);
+
+        // Check that the sizes are greater than zero
+        assert!(sizes[0] > 0);
+
+        let _ = DB::destroy(&Options::default(), &path);
+    }
+}
